=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2016-05-10 06:21:37 +0000
+++ CMakeLists.txt	2016-05-10 09:38:09 +0000
@@ -86,6 +86,7 @@
 
 add_subdirectory(src)
 add_subdirectory(tests)
+add_subdirectory(demo)
 
 enable_coverage_report(
     TARGETS

=== added directory 'demo'
=== added file 'demo/CMakeLists.txt'
--- demo/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ demo/CMakeLists.txt	2016-05-10 09:38:09 +0000
@@ -0,0 +1,5 @@
+
+add_definitions(-DBOOST_THREAD_VERSION=4)
+
+add_executable(provider-test provider-test.cpp)
+target_link_libraries(provider-test storage-framework-provider)

=== added file 'demo/provider-test.cpp'
--- demo/provider-test.cpp	1970-01-01 00:00:00 +0000
+++ demo/provider-test.cpp	2016-05-10 09:38:09 +0000
@@ -0,0 +1,86 @@
+
+#include <unity/storage/provider/ProviderBase.h>
+#include <unity/storage/provider/Server.h>
+
+#include <stdexcept>
+
+using namespace unity::storage::provider;
+
+class MyProvider : public ProviderBase
+{
+public:
+    MyProvider();
+
+    boost::future<std::vector<Item>> roots() override;
+    boost::future<std::tuple<std::vector<Item>,std::string>> list(
+        std::string const& item_id, std::string const& page_token) override;
+    boost::future<std::vector<Item>> lookup(
+        std::string const& parent_id, std::string const& name) override;
+    boost::future<Item> metadata(std::string const& item_id) override;
+};
+
+MyProvider::MyProvider()
+{
+}
+
+boost::future<std::vector<Item>> MyProvider::roots()
+{
+    std::vector<Item> roots = {
+        {"root_id", "", "Root", "etag", ItemType::root, {}},
+    };
+    return boost::make_ready_future<std::vector<Item>>(roots);
+}
+
+boost::future<std::tuple<std::vector<Item>,std::string>> MyProvider::list(
+    std::string const& item_id, std::string const& page_token)
+{
+    if (item_id != "root_id")
+    {
+        return boost::make_exceptional_future<std::tuple<std::vector<Item>,std::string>>(std::runtime_error("unknown folder"));
+    }
+    if (page_token != "")
+    {
+        return boost::make_exceptional_future<std::tuple<std::vector<Item>,std::string>>(std::runtime_error("unknown page token"));
+    }
+    std::vector<Item> children = {
+        {"child_id", "root_id", "Child", "etag", ItemType::file, {}}
+    };
+    boost::promise<std::tuple<std::vector<Item>,std::string>> p;
+    p.set_value(std::make_tuple(children, std::string()));
+    return p.get_future();
+}
+
+boost::future<std::vector<Item>> MyProvider::lookup(
+    std::string const& parent_id, std::string const& name)
+{
+    if (parent_id != "root_id" || name != "Child")
+    {
+        return boost::make_exceptional_future<std::vector<Item>>(std::runtime_error("file not found"));
+    }
+    std::vector<Item> children = {
+        {"child_id", "root_id", "Child", "etag", ItemType::file, {}}
+    };
+    return boost::make_ready_future<std::vector<Item>>(children);
+}
+
+boost::future<Item> MyProvider::metadata(std::string const& item_id)
+{
+    if (item_id == "root_id")
+    {
+        Item metadata{"root_id", "", "Root", "etag", ItemType::root, {}};
+        return boost::make_ready_future<Item>(metadata);
+    }
+    else if (item_id == "child_id")
+    {
+        Item metadata{"child_id", "root_id", "Child", "etag", ItemType::file, {}};
+        return boost::make_ready_future<Item>(metadata);
+    }
+    return boost::make_exceptional_future<Item>(std::runtime_error("no such file"));
+}
+
+int main(int argc, char **argv)
+{
+    Server<MyProvider> server;
+    server.init(argc, argv);
+    server.run();
+}

=== modified file 'include/unity/storage/provider/ProviderBase.h'
--- include/unity/storage/provider/ProviderBase.h	2016-05-10 06:21:37 +0000
+++ include/unity/storage/provider/ProviderBase.h	2016-05-10 09:38:09 +0000
@@ -20,7 +20,7 @@
     root,
 };
 
-struct Item
+struct __attribute__((visibility("default"))) Item
 {
     std::string item_id;
     std::string parent_id;
@@ -31,7 +31,8 @@
     std::map<std::string,std::string> metadata;
 };
 
-class ProviderBase {
+class __attribute__((visibility("default"))) ProviderBase
+{
 public:
     ProviderBase();
     virtual ~ProviderBase();

=== modified file 'include/unity/storage/provider/Server.h'
--- include/unity/storage/provider/Server.h	2016-05-10 06:21:37 +0000
+++ include/unity/storage/provider/Server.h	2016-05-10 09:38:09 +0000
@@ -18,7 +18,8 @@
 
 class ProviderBase;
 
-class ServerBase {
+class __attribute__((visibility("default"))) ServerBase
+{
 public:
     ServerBase();
     virtual ~ServerBase();
@@ -34,7 +35,8 @@
 };
 
 template <typename T>
-class Server : public ServerBase {
+class Server : public ServerBase
+{
 public:
     using ServerBase::ServerBase;
 protected:

=== modified file 'include/unity/storage/provider/internal/ProviderInterface.h'
--- include/unity/storage/provider/internal/ProviderInterface.h	2016-05-10 06:21:37 +0000
+++ include/unity/storage/provider/internal/ProviderInterface.h	2016-05-10 09:38:09 +0000
@@ -24,7 +24,7 @@
 {
 };
 
-class ProviderInterface : public QObject, private QDBusContext
+class ProviderInterface : public QObject, protected QDBusContext
 {
     Q_OBJECT
 

=== modified file 'include/unity/storage/provider/internal/dbusmarshal.h'
--- include/unity/storage/provider/internal/dbusmarshal.h	2016-05-10 06:21:37 +0000
+++ include/unity/storage/provider/internal/dbusmarshal.h	2016-05-10 09:38:09 +0000
@@ -14,14 +14,11 @@
 struct Item;
 
 QDBusArgument& operator<<(QDBusArgument& argument, Item const& item);
-
-namespace internal
-{
-
-QVariant marshal_item(Item const& item);
-QVariant marshal_item_list(std::vector<Item> const& list);
-
-}
+QDBusArgument const& operator>>(QDBusArgument const& argument, Item& item);
+
+QDBusArgument& operator<<(QDBusArgument& argument, std::vector<Item> const& items);
+QDBusArgument const& operator>>(QDBusArgument const& argument, std::vector<Item>& items);
+
 }
 }
 }

=== modified file 'src/provider/CMakeLists.txt'
--- src/provider/CMakeLists.txt	2016-05-10 06:21:37 +0000
+++ src/provider/CMakeLists.txt	2016-05-10 09:38:09 +0000
@@ -14,6 +14,8 @@
   internal/ProviderInterface.cpp
   internal/Handler.cpp
   internal/dbusmarshal.cpp
+  ${CMAKE_SOURCE_DIR}/include/unity/storage/provider/internal/ProviderInterface.h
+  ${CMAKE_SOURCE_DIR}/include/unity/storage/provider/internal/Handler.h
   ${generated_files})
 
 set_target_properties(storage-framework-provider PROPERTIES AUTOMOC TRUE)

=== modified file 'src/provider/Server.cpp'
--- src/provider/Server.cpp	2016-05-10 06:21:37 +0000
+++ src/provider/Server.cpp	2016-05-10 09:38:09 +0000
@@ -1,6 +1,7 @@
 #include <unity/storage/provider/Server.h>
 #include <unity/storage/provider/ProviderBase.h>
 #include <unity/storage/provider/internal/ProviderInterface.h>
+#include <unity/storage/provider/internal/dbusmarshal.h>
 #include "provideradaptor.h"
 
 #include <QCoreApplication>
@@ -28,11 +29,13 @@
     // this instance is managed by Qt's parent/child memory management
     new ProviderAdaptor(interface.get());
 
+    qDBusRegisterMetaType<Item>();
+    qDBusRegisterMetaType<std::vector<Item>>();
     auto bus = QDBusConnection::sessionBus();
     bus.registerObject("/provider", interface.get());
 
     // TODO: claim bus name
-    qDebug() << "Bus unique name:" << bus.name();
+    qDebug() << "Bus unique name:" << bus.baseService();
 }
 
 void ServerBase::run()

=== modified file 'src/provider/internal/ProviderInterface.cpp'
--- src/provider/internal/ProviderInterface.cpp	2016-05-10 06:21:37 +0000
+++ src/provider/internal/ProviderInterface.cpp	2016-05-10 09:38:09 +0000
@@ -53,7 +53,7 @@
             auto f = provider->roots();
             return f.then([=](decltype(f) f) -> QDBusMessage {
                     auto roots = f.get();
-                    return message.createReply(marshal_item_list(roots));
+                    return message.createReply(QVariant::fromValue(roots));
                 });
         });
     return {};
@@ -68,7 +68,7 @@
                     string next_token;
                     tie(children, next_token) = f.get();
                     return message.createReply({
-                            marshal_item_list(children),
+                            QVariant::fromValue(children),
                             QVariant(QString::fromStdString(next_token)),
                         });
                 });
@@ -82,7 +82,7 @@
             auto f = provider->lookup(parent_id.toStdString(), name.toStdString());
             return f.then([=](decltype(f) f) -> QDBusMessage {
                     auto items = f.get();
-                    return message.createReply(marshal_item_list(items));
+                    return message.createReply(QVariant::fromValue(items));
                 });
         });
     return {};
@@ -94,7 +94,7 @@
             auto f = provider->metadata(item_id.toStdString());
             return f.then([=](decltype(f) f) -> QDBusMessage {
                     auto item = f.get();
-                    return message.createReply(marshal_item(item));
+                    return message.createReply(QVariant::fromValue(item));
                 });
         });
     return {};

=== modified file 'src/provider/internal/dbusmarshal.cpp'
--- src/provider/internal/dbusmarshal.cpp	2016-05-06 13:05:39 +0000
+++ src/provider/internal/dbusmarshal.cpp	2016-05-10 09:38:09 +0000
@@ -2,6 +2,7 @@
 #include <unity/storage/provider/ProviderBase.h>
 
 #include <cstdint>
+#include <stdexcept>
 
 using namespace std;
 
@@ -32,29 +33,27 @@
     return argument;
 }
 
-namespace internal
-{
-
-QVariant marshal_item(Item const& item)
-{
-    QDBusArgument argument;
-    argument << item;
-    return argument.asVariant();
+QDBusArgument const& operator>>(QDBusArgument const& argument, Item& item)
+{
+    throw std::runtime_error("Item decode not implemented");
 }
 
-QVariant marshal_item_list(std::vector<Item> const& list)
+QDBusArgument& operator<<(QDBusArgument& argument, std::vector<Item> const& items)
 {
-    QDBusArgument argument;
     argument.beginArray(qMetaTypeId<Item>());
-    for (auto const& item : list)
+    for (auto const& item : items)
     {
         argument << item;
     }
     argument.endArray();
-    return argument.asVariant();
-}
-
-}
+    return argument;
+}
+
+QDBusArgument const& operator>>(QDBusArgument const& argument, std::vector<Item>& items)
+{
+    throw std::runtime_error("std::vector<Item> decode not implemented");
+}
+
 }
 }
 }

